package dev.kske.chess.board;

import java.util.ArrayList;
import java.util.List;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>King.java</strong><br>
 * Created: <strong>01.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class King extends Piece {

	public King(Color color, Board board) {
		super(color, board);
	}

	@Override
	public boolean isValidMove(Move move) {
		return move.xDist <= 1 && move.yDist <= 1 && isFreePath(move);
	}

	@Override
	protected List<Move> getPseudolegalMoves(Position pos) {
		List<Move> moves = new ArrayList<>();
		for (int i = Math.max(0, pos.x - 1); i < Math.min(8, pos.x + 2); i++)
			for (int j = Math.max(0, pos.y - 1); j < Math.min(8, pos.y + 2); j++)
				if (i != pos.x || j != pos.y) {
					Move move = new Move(pos, new Position(i, j));
					if (board.getDest(move) == null || board.getDest(move).getColor() != getColor()) moves.add(move);
				}
		return moves;
	}

	@Override
	public Type getType() { return Type.KING; }
}
