package dev.kske.chess.game;

import dev.kske.chess.board.Board;
import dev.kske.chess.board.Move;
import dev.kske.chess.board.Piece;
import dev.kske.chess.board.Piece.Color;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>AIPlayer.java</strong><br>
 * Created: <strong>06.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class AIPlayer extends Player {

	private Move bestMove;
	private int		count;

	public AIPlayer(Board board, Color color) {
		super(board, color);
	}

	@Override
	public void requestMove() {
		count = 0;
		findBestMove(board, color, 0);
		System.out.println("Moved processes: " + count);
		game.onMove(this, bestMove);
	}

	private int findBestMove(Board board, Color color, int depth) {
		int bestValue = Integer.MIN_VALUE;
		for (Move move : board.getMoves(color)) {
			++count;
			Piece	capturePiece	= board.move(move);
			int		teamValue		= board.evaluate(color);
			int		enemyValue		= board.evaluate(color.opposite());
			int		valueChange		= teamValue - enemyValue;

			if (depth <= 3 && valueChange >= 0) valueChange -= findBestMove(board, color.opposite(), depth + 1);

			if (valueChange > bestValue) {
				bestValue = valueChange;
				if (depth == 0) bestMove = move;
			}

			board.revert(move, capturePiece);
		}
		return bestValue;
	}
}
