package dev.kske.chess.game;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import dev.kske.chess.board.Board;
import dev.kske.chess.board.Move;
import dev.kske.chess.board.Piece.Color;
import dev.kske.chess.board.Position;
import dev.kske.chess.ui.BoardPanel;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>NaturalPlayer.java</strong><br>
 * Created: <strong>06.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class NaturalPlayer extends Player {

	private boolean moveRequested;

	public NaturalPlayer(Board board, Color color, BoardPanel boardPanel) {
		super(board, color);
		moveRequested = false;
		boardPanel.addMouseListener(new MouseAdapter() {

			private Position pos;

			@Override
			public void mousePressed(MouseEvent evt) {
				if (!moveRequested) return;
				if (pos == null) {
					pos = new Position(evt.getPoint().x / boardPanel.getTileSize(),
							evt.getPoint().y / boardPanel.getTileSize());

					if (board.get(pos) != null && board.get(pos).getColor() == color)
						boardPanel.displayMoves(board.getMoves(pos));
					else pos = null;
				} else {
					Position dest = new Position(evt.getPoint().x / boardPanel.getTileSize(),
							evt.getPoint().y / boardPanel.getTileSize());

					boardPanel.clearDisplayMoves();
					moveRequested = false;
					game.onMove(NaturalPlayer.this, new Move(pos, dest));
					pos				= null;
				}
			}
		});
	}

	@Override
	public void requestMove() {
		moveRequested = true;
	}
}
