package dev.kske.chess.ui;

import java.awt.FlowLayout;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JDialog;

import dev.kske.chess.board.Piece.Color;
import dev.kske.chess.game.AIPlayer;
import dev.kske.chess.game.Game;
import dev.kske.chess.game.NaturalPlayer;
import dev.kske.chess.game.Player;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>GameModeDialog.java</strong><br>
 * Created: <strong>06.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class GameModeDialog extends JDialog {

	private static final long serialVersionUID = 5470026233924735607L;

	/**
	 * Create the dialog.
	 */
	public GameModeDialog(BoardPanel boardPanel) {
		super();
		setModal(true);
		setTitle("Game Mode Selection");
		setBounds(100, 100, 231, 133);
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		getContentPane().setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));

		JButton btnNatural = new JButton("Game against natural opponent");
		btnNatural.addActionListener((evt) -> {
			Map<Color, Player> players = new HashMap<>();
			players.put(Color.WHITE, new NaturalPlayer(boardPanel.getBoard(), Color.WHITE, boardPanel));
			players.put(Color.BLACK, new NaturalPlayer(boardPanel.getBoard(), Color.BLACK, boardPanel));
			new Game(players, boardPanel.getBoard(), boardPanel).start();
			dispose();
		});
		getContentPane().add(btnNatural);

		JButton btnAI = new JButton("Game against AI");
		btnAI.addActionListener((evt) -> {
			Map<Color, Player> players = new HashMap<>();
			players.put(Color.WHITE, new NaturalPlayer(boardPanel.getBoard(), Color.WHITE, boardPanel));
			players.put(Color.BLACK, new AIPlayer(boardPanel.getBoard(), Color.BLACK));
			startGame(players, boardPanel);
		});
		getContentPane().add(btnAI);

		JButton btnAI2 = new JButton("AI against AI");
		btnAI2.addActionListener((evt) -> {
			Map<Color, Player> players = new HashMap<>();
			players.put(Color.WHITE, new AIPlayer(boardPanel.getBoard(), Color.WHITE));
			players.put(Color.BLACK, new AIPlayer(boardPanel.getBoard(), Color.BLACK));
			startGame(players, boardPanel);
		});
		getContentPane().add(btnAI2);
	}

	private void startGame(Map<Color, Player> players, BoardPanel boardPanel) {
		new Game(players, boardPanel.getBoard(), boardPanel).start();
		dispose();
	}
}
